@@ -7,7 +7,7 @@
 #
 # -----------------------------------------------------------------------------
 '''
-FreeType high-level python API 
+FreeType high-level python API
 
 This the bindings for the high-level API of FreeType (that must be installed
 somewhere on your system).
@@ -16,15 +16,20 @@ Note:
   C Library will be searched using the ctypes.util.find_library. However, this
   search might fail. In such a case (or for other reasons), you can specify the
   FT_library_filename before importing the freetype library and freetype will use
-  the specified one. 
+  the specified one.
 '''
+import sys
 from ctypes import *
-from ft_types import *
-from ft_enums import *
-from ft_errors import *
-from ft_structs import *
+from freetype.ft_types import *
+from freetype.ft_enums import *
+from freetype.ft_errors import *
+from freetype.ft_structs import *
 import ctypes.util
 
+# Hack to get unicode class in python3
+PY3 = sys.version_info[0] == 3
+if PY3: unicode = str
+
 
 __dll__    = None
 __handle__ = None
@@ -35,7 +40,7 @@ if not FT_Library_filename:
     except OSError:
         __dll__ = None
 if not FT_Library_filename and not __dll__:
-    raise RuntimeError, 'Freetype library not found'
+    raise RuntimeError('Freetype library not found')
 if not __dll__:
   __dll__ = ctypes.CDLL(FT_Library_filename)
 
@@ -224,8 +229,8 @@ def set_lcd_filter_weights(a,b,c,d,e):
         error = FT_Library_SetLcdFilterWeights(library, weights)
         if error: raise FT_Exception(error)
     else:
-        raise RuntimeError, \
-              'set_lcd_filter_weights require freetype > 2.4.0'
+        raise RuntimeError(
+              'set_lcd_filter_weights require freetype > 2.4.0')
 
 
 
@@ -316,8 +321,8 @@ class GlyphMetrics( object ):
 
     width = property( lambda self: self._FT_Glyph_Metrics.width,
        doc = '''The glyph's width.''' )
-    
-    height = property( lambda self: self._FT_Glyph_Metrics.heigth,
+
+    height = property( lambda self: self._FT_Glyph_Metrics.height,
        doc = '''The glyph's height.''' )
 
     horiBearingX = property( lambda self: self._FT_Glyph_Metrics.horiBearingX,
@@ -517,7 +522,7 @@ class Bitmap(object):
     pixel_mode = property(lambda self: self._FT_Bitmap.pixel_mode,
            doc = '''The pixel mode, i.e., how pixel bits are stored. See
                     FT_Pixel_Mode for possible values.''')
-  
+
     palette_mode = property(lambda self: self._FT_Bitmap.palette_mode,
              doc ='''This field is intended for paletted pixel modes; it
                      indicates how the palette is stored. Not used currently.''')
@@ -630,7 +635,7 @@ class Outline( object ):
         :param charmap: a FT_Outline
         '''
         self._FT_Outline = outline
-        
+
     n_contours = property(lambda self: self._FT_Outline.n_contours)
     def _get_contours(self):
         n = self._FT_Outline.n_contours
@@ -669,7 +674,7 @@ class Outline( object ):
               argument to the SCANMODE instruction).
 
               Bits 3 and 4 are reserved for internal purposes.''')
-        
+
     flags = property(lambda self: self._FT_Outline.flags,
       doc = '''A set of bit flags used to characterize the outline and give
                hints to the scan-converter and hinter on how to
@@ -833,18 +838,18 @@ class Glyph( object ):
           can compute the width and height of the glyph image (be it in integer
           or 26.6 pixels) as:
 
-          width  = bbox.xMax - bbox.xMin;                                  
-          height = bbox.yMax - bbox.yMin;                                  
+          width  = bbox.xMax - bbox.xMin;
+          height = bbox.yMax - bbox.yMin;
 
           Note also that for 26.6 coordinates, if 'bbox_mode' is set to
           FT_GLYPH_BBOX_GRIDFIT, the coordinates will also be grid-fitted, which
           corresponds to:
 
           bbox.xMin = FLOOR(bbox.xMin);
-          bbox.yMin = FLOOR(bbox.yMin);                                    
-          bbox.xMax = CEILING(bbox.xMax);                                  
-          bbox.yMax = CEILING(bbox.yMax);                                  
-        
+          bbox.yMin = FLOOR(bbox.yMin);
+          bbox.xMax = CEILING(bbox.xMax);
+          bbox.yMax = CEILING(bbox.yMax);
+
           To get the bbox in pixel coordinates, set 'bbox_mode' to
           FT_GLYPH_BBOX_TRUNCATE.
 
@@ -1040,13 +1045,13 @@ class Face( object ):
         face = FT_Face( )
         self._FT_Face = None
         #error = FT_New_Face( library, filename, 0, byref(face) )
-        u_filename = c_char_p(filename)
+        u_filename = c_char_p(filename.encode())
         error = FT_New_Face( library, u_filename, index, byref(face) )
         if error: raise FT_Exception( error )
         self._filename = filename
         self._index = index
         self._FT_Face = face
-    
+
     def __del__( self ):
         '''
         Discard  face object, as well as all of its child slots and sizes.
@@ -1054,11 +1059,36 @@ class Face( object ):
         if self._FT_Face is not None:
             FT_Done_Face( self._FT_Face )
 
+
+    def attach_file( self, filename ):
+        '''
+        Attach data to a face object. Normally, this is used to read
+        additional information for the face object. For example, you can attach
+        an AFM file that comes with a Type 1 font to get the kerning values and
+        other metrics.
+
+        :param filename: Filename to attach
+
+        **Note**
+
+        The meaning of the 'attach' (i.e., what really happens when the new
+        file is read) is not fixed by FreeType itself. It really depends on the
+        font format (and thus the font driver).
+
+        Client applications are expected to know what they are doing when
+        invoking this function. Most drivers simply do not implement file
+        attachments.
+        '''
+
+        error = FT_Attach_File( self._FT_Face, filename)
+        if error: raise FT_Exception( error)
+
+
     def set_char_size( self, width=0, height=0, hres=72, vres=72 ):
         '''
         This function calls FT_Request_Size to request the nominal size (in
         points).
-        
+
         :param float width: The nominal width, in 26.6 fractional points.
 
         :param float height: The nominal height, in 26.6 fractional points.
@@ -1135,8 +1165,8 @@ class Face( object ):
           correspond to the internal indices used within the file. This is done
           to ensure that value 0 always corresponds to the 'missing glyph'.
         '''
-        if type( charcode ) in (str,unicode):
-            charcode = ord( charcode )
+        if isinstance(charcode, (str,unicode)):
+            charcode = ord(charcode)
         return FT_Get_Char_Index( self._FT_Face, charcode )
 
     def get_first_char( self ):
@@ -1171,7 +1201,7 @@ class Face( object ):
         :param agindex: Glyph index of next character code. 0 if charmap is empty.
 
         **Note**:
-  
+
           You should use this function with FT_Get_First_Char to walk over all
           character codes available in a given charmap. See the note for this
           function for a simple code example.
@@ -1288,7 +1318,7 @@ class Face( object ):
         :param flags: A set of bit flags similar to those used when calling
                       FT_Load_Glyph, used to determine what kind of advances
                       you need.
-                    
+
         :return: The advance value, in either font units or 16.16 format.
 
                  If FT_LOAD_VERTICAL_LAYOUT is set, this is the vertical
@@ -1300,7 +1330,7 @@ class Face( object ):
         error = FT_Get_Advance( self._FT_Face, gindex, flags, byref(padvance) )
         if error: raise FT_Exception( error )
         return padvance.value
-        
+
 
 
     def get_kerning( self, left, right, mode = FT_KERNING_DEFAULT ):
@@ -1308,7 +1338,7 @@ class Face( object ):
         Return the kerning vector between two glyphs of a same face.
 
         :param left: The index of the left glyph in the kern pair.
-        
+
         :param right: The index of the right glyph in the kern pair.
 
         :param mode: See FT_Kerning_Mode for more information. Determines the scale
@@ -1336,7 +1366,7 @@ class Face( object ):
         'TrueType', 'Type 1', 'BDF', ‘PCF', ‘Type 42', ‘CID Type 1', ‘CFF',
         'PFR', and ‘Windows FNT'.
         '''
-        
+
         return FT_Get_X11_Font_Format( self._FT_Face )
 
 
@@ -1427,7 +1457,7 @@ class Face( object ):
                doc = '''True whenever a face object contains a font face that
                         contains fixed-width (or 'monospace', 'fixed-pitch',
                         etc.) glyphs.''')
-    
+
     def _has_fixed_sizes( self ):
         return bool( self.face_flags & FT_FACE_FLAG_FIXED_SIZES )
     has_fixed_sizes = property( _has_fixed_sizes,
@@ -1769,7 +1799,7 @@ class Stroker( object ):
         error = FT_Stroker_BeginSubPath( self._FT_Stroker, to, _open )
         if error: raise FT_Exception( error )
 
-    
+
     def end_subpath( self ):
         '''
         Close the current sub-path in the stroker.
@@ -1783,7 +1813,7 @@ class Stroker( object ):
         error = FT_Stroker_EndSubPath( self._FT_Stroker)
         if error: raise FT_Exception( error )
 
-    
+
     def line_to( self, to ):
         '''
         'Draw' a single line segment in the stroker's current sub-path, from
@@ -1799,7 +1829,7 @@ class Stroker( object ):
         error = FT_Stroker_LineTo( self._FT_Stroker, to )
         if error: raise FT_Exception( error )
 
-        
+
     def conic_to( self, control, to ):
         '''
         'Draw' a single quadratic Bezier in the stroker's current sub-path,
@@ -1816,7 +1846,7 @@ class Stroker( object ):
         '''
         error = FT_Stroker_ConicTo( self._FT_Stroker, control, to )
         if error: raise FT_Exception( error )
-        
+
 
     def cubic_to( self, control1, control2, to ):
         '''
@@ -1836,15 +1866,15 @@ class Stroker( object ):
         '''
         error = FT_Stroker_CubicTo( self._FT_Stroker, control1, control2, to )
         if error: raise FT_Exception( error )
-        
-        
+
+
     def get_border_counts( self, border ):
         '''
         Call this function once you have finished parsing your paths with the
         stroker. It returns the number of points and contours necessary to
         export one of the 'border' or 'stroke' outlines generated by the
         stroker.
-        
+
         :param border: The border index.
 
         :return: number of points, number of contours
@@ -1855,7 +1885,7 @@ class Stroker( object ):
                                     byref(anum_points), byref(anum_contours) )
         if error: raise FT_Exception( error )
         return anum_points.value, anum_contours.value
-                                            
+
 
     def export_border( self , border, outline ):
         '''
@@ -1868,7 +1898,7 @@ class Stroker( object ):
         :param border:  The border index.
 
         :param outline: The target outline.
-            
+
         **Note**:
 
           Always call this function after get_border_counts to get sure that
